//ZylUdpGPSReceiver
//Copyright by Zyl Soft 2003 - 2012
//For Indy 10

unit ZylUdp10GPSReceiver;

{$I IdCompilerDefines.inc}

//{$C-}
interface

uses
   ZylCustomGPSReceiver, SysUtils, Windows, Classes, Math
   , IdUDPServer, IdBaseComponent, IdComponent, IdUDPBase, IdSocketHandle, IdGlobal
   {$ifndef NODF}
   , Dialogs, Forms, Controls
   {$endif}
   {$ifdef DELPHI6UP}
   , DateUtils
   {$endif}
   {$ifdef RADSTUDIOXE2}
   , Vcl.Dialogs, Vcl.Forms, Vcl.Controls
   {$endif};

type
  TConnectEvent = procedure(Sender: TObject; const Port: Integer) of object;

  //Receiver class
  TZylUdp10GPSReceiver = class(TZylCustomGPSReceiver)
  private
    { Private declarations }
    FPort: Integer;
    FRemoteHost: AnsiString;

    FSocket: TIdUDPServer;

    FOnConnect: TConnectEvent;
    FOnDisconnect: TConnectEvent;

    procedure SetPortA(Value: Integer);

    {$ifdef RADSTUDIO2009UP}
    procedure ServerSocketUDPRead(Sender: TIdUDPListenerThread; AData: TBytes; ABinding: TIdSocketHandle);
    {$else}
    procedure ServerSocketUDPRead(Sender: TObject; AData: TBytes; ABinding: TIdSocketHandle);
    {$endif}
  protected
    { Protected declarations }

  public
    { Public declarations }

    property RemoteHost: AnsiString read FRemoteHost write FRemoteHost;

    //constructor
    constructor Create(AOwner: TComponent); override;
    //destructor
    destructor Destroy();override;

    //starts communication
    procedure Open();
    //stops communication
    procedure Close();
    function Send(const str: String): DWORD;
    procedure UploadWaypoint(const WayPoint: TWayPoint);
    //uploads route
    procedure UploadRoute(const Route: TRoute);
  published
    { Published declarations }
    property Port: Integer read FPort write SetPortA default 10000;

    property ForceCheckSum;
    {$ifndef CPPB5}
    property NMEALog;
    property LogDateTime;
    property LogFile;
    {$endif}
    property Commands;

    property OnConnect: TConnectEvent read FOnConnect write FOnConnect;
    property OnDisconnect: TConnectEvent read FOnDisconnect write FOnDisconnect;
    //fires when new data was received
    property OnReceive;
    property OnSend;
    property OnLatitudeChange;
    property OnLongitudeChange;
    property OnAltitudeChange;
    property OnSpeedChange;
    property OnHeadingChange;
    property OnPosition2DChange;
    property OnPosition3DChange;
    property OnSatelliteReceive;
    property OnSatellitesReceive;
    property OnArrival;
    property OnNewCommand;
    property OnUnknownCommand;
    property OnWayPointReceive;
    property OnActiveRouteReceive;    
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Zyl Soft', [TZylUdp10GPSReceiver]);
end;

{ TZylUdp10GPSReceiver }

constructor TZylUdp10GPSReceiver.Create(AOwner: TComponent);
begin
  inherited;

  FPort := 10000;
  FRemoteHost := '';

  FOnConnect := nil;
  FOnDisconnect := nil;

  FSocket := TIdUDPServer.Create(Self);
  FSocket.DefaultPort := FPort;
  FSocket.OnUDPRead := ServerSocketUDPRead;
end;

procedure TZylUdp10GPSReceiver.SetPortA(Value: Integer);
begin
  try
    if FPort <> Value then
    begin
      FPort := Value;
      if FSocket.Active then
      begin
        Self.Open;
      end;
    end;
  except
    raise EZylGPSReceiverException.Create('Invalid port ' + IntToStr(Value) + '.');
  end;
end;

procedure TZylUdp10GPSReceiver.Open();
begin
  Self.Close;
  ResetPosition;
  ResetActiveRoute;  
  ClearWayPoints;
  ResetNavigationInfo;
  ResetReceiverStatus;


  FSocket.DefaultPort := FPort;
  FSocket.Active := True;

  if Assigned(FOnConnect) and FSocket.Active then
    FOnConnect(Self, FPort);
end;

procedure TZylUdp10GPSReceiver.Close();
begin
  if FSocket.Active then
  begin
    if Assigned(FOnDisconnect) then
      FOnDisconnect(Self, FPort);
    try
      if FSocket.Active then
        FSocket.Active := False;
    except
    end;
  end;
  ResetPosition;
  ResetActiveRoute;
  ClearWayPoints;
  ResetNavigationInfo;
  ResetReceiverStatus;  
end;

destructor TZylUdp10GPSReceiver.Destroy;
begin
  if FSocket.Active then
  begin
    FSocket.Active := False;
    FSocket.Free;
    FSocket := nil;
  end;
  inherited;
end;

procedure TZylUdp10GPSReceiver.ServerSocketUDPRead(Sender: TIdUDPListenerThread; AData: TBytes; ABinding: TIdSocketHandle);
var
  strPacket: AnsiString;
begin
  try
    strPacket := BytesToString(Adata);
    FeedGPSReceiver(strPacket);
    if Trim(ABinding.PeerIP) <> '' then
      FRemoteHost := ABinding.PeerIP;
  except
  end;
end;

function TZylUdp10GPSReceiver.Send(const str: String): DWORD;
begin
  Result := 0; 
  if Trim(FRemoteHost) <> '' then
  begin
    try
      FSocket.Send(FRemoteHost, FPort, str);
      if Assigned(OnSend) then
      begin
        OnSend(Self, str);
      end;
      Result := Length(str);
    except
    end;
  end;
end;

procedure TZylUdp10GPSReceiver.UploadRoute(const Route: TRoute);
begin
  Send(RouteToNMEA(Route));
end;

procedure TZylUdp10GPSReceiver.UploadWaypoint(const WayPoint: TWayPoint);
begin
  Send(WayPointToNMEA(WayPoint));
end;

end.


